%% tclp__main.pl
%% Copyright (C) 2001,2002 INRIA Emmanuel Coquery

%% This library is free software; you can redistribute it and/or
%% modify it under the terms of the GNU Lesser General Public
%% License as published by the Free Software Foundation; either
%% version 2.1 of the License, or (at your option) any later version.

%% This library is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.

%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                           %%
%%                                  TCLP                                     %%
%%                                                                           %%
%%         Emmanuel Coquery, projet Contraintes, INRIA Rocquencourt          %%
%%                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Header: /home/beaune/contraintes/coquery/cvs/contraintes/tclpchr/src/tclp__main.pl,v 1.22 2002/09/19 15:35:44 coquery Exp $

:- use_module(tclp).
:- use_module(tclp__utils).
:- use_module(tclp__errors).
:- use_module(tclp__files).
:- use_module(sicstus_modules).

:- use_module(library(lists)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Process a file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% All the files to process
:- dynamic pl_files/1, typ_files/1.

pl_files([]).
typ_files([]).

%% add a prolog_file to all prolog files to type_check
add_pl_file(File) :-
        pl_files(CurrentFiles),
        append(CurrentFiles, [File], NewFiles),
        retractall(pl_files(_)),
        asserta(pl_files(NewFiles)).

%% add a typ_file to all typ files to load
add_typ_file(File) :-
        typ_files(CurrentFiles),
        append(CurrentFiles, [File], NewFiles),
        retractall(typ_files(_)),
        asserta(typ_files(NewFiles)).

%% Process all arguments
process_args([]).
process_args([ '--prolog' , Prolog | Rest ]) :- !,
        verbose(debug_main,['processing --prolog ', Prolog, ' | ', Rest]),
        (
          Prolog == sicstus
        ->
          enable_sicstus
        ;
          true
        ),
        process_args(Rest).
process_args([ '-l' | Rest ]) :- !,
        verbose(debug_main,['processing -l ', ' | ', Rest]),
        tclp__set_functor_labeling(true),
        process_args(Rest).
process_args([ '-nl' | Rest ]) :- !,
        verbose(debug_main,['processing -nl ', ' | ', Rest]),
        tclp__set_functor_labeling(false),
        process_args(Rest).
process_args([ '-I' , Dir | Rest ]) :- !,
        verbose(debug_main,['processing -I ', Dir, ' | ', Rest]),
        tclp__add_search_directory(Dir),!,
        process_args(Rest).
process_args([ '--stdlib' , StdLib | Rest ]) :-
        verbose(debug_main,['processing --stdlib ', StdLib, ' | ', Rest]),
        tclp__set_stdlib_name(StdLib),!,
        process_args(Rest).
process_args([ '-v', VerboseLevel | Rest ]) :-
        tclp__enable_verbose(VerboseLevel),!,
        process_args(Rest).
process_args([ '-nv', VerboseLevel | Rest ]) :-
        tclp__disable_verbose(VerboseLevel),!,
        process_args(Rest).
process_args([ '--help' | _ ]) :-
        print_usage, halt(1).
process_args([ '-i' | Rest ]) :- !,
        verbose(debug_main,['processing -i', ' | ', Rest]),
        tclp__set_inference(true),
        process_args(Rest).
process_args([ '--backward_compatibility' | Rest ]) :- !,
        verbose(debug_main,['processing --backward_compatibility', ' | ',
                            Rest]),
        tclp__declarations:enable_backward_compatibility,
        process_args(Rest).
process_args([ '--type_inference' | Rest ]) :- !,
        verbose(debug_main,['processing --type_inference', ' | ', Rest]),
        tclp__set_inference(true),
        process_args(Rest).
process_args([ '--type_check' | Rest ]) :- !,
        verbose(debug_main,['processing --type_check', ' | ', Rest]),
        tclp__set_inference(false),
        process_args(Rest).
process_args([ '--heuristic' | Rest ]) :- !,
        verbose(debug_main,['processing --heuristic', ' | ', Rest]),
        tclp__use_labeling_heuristic(true),
        process_args(Rest).
process_args([ '--look_ahead' , LA | Rest ]) :- !,
        verbose(debug_main,['processing --look_ahead ', LA, ' | ', Rest]),
        tclp__labeling_look_ahead(LA),
        process_args(Rest).
process_args([ Arg | Rest ]) :-
        verbose(debug_main,['processing ', Arg, ' | ', Rest]),
        chop_ext(Arg,'.pl',_),!,
        add_pl_file(Arg),
        process_args(Rest).
process_args([ Arg | Rest ]) :-
        verbose(debug_main,['processing ', Arg, ' | ', Rest]),
        chop_ext(Arg, '.typ',_),!,
        add_typ_file(Arg),
        process_args(Rest).
process_args([ Arg | _ ]) :-
        print_usage,
        print_error(['Illegal argument : ',Arg]).

%% Run the type checker according to the given arguments
run(Args) :-
        verbose(debug_main,'starting cmd arg parsing'),
        process_args(Args),
        verbose(debug_main,'cmd line ok'),
        tclp:stdlib(StdLib),
        verbose(actions,[ 'Loading standard type library (',StdLib,')' ]),
        once(chrono([ 'loading standard type library (',StdLib,')' ],
                    tclp__process_typ([StdLib]))),
        typ_files(TypFiles),
        (
          TypFiles \= []
        ->
          verbose(actions,['Loading : ',TypFiles]),
          once(chrono(['loading : ',TypFiles,' ...'],
                      tclp__process_typ(TypFiles)))
        ;
          true
        ),
        pl_files(PlFiles),
        tclp__process_pl(PlFiles),
        verbose(debug_main,'Finished')
        ;
        print_error('Failed').

%% Usage message
print_usage :-
        nl,write('usage :'),nl,nl,
        write('tclp [ -I dir ] [ [-v|-nv] mode ] [ --help ] [ [file.pl|file.typ] ... ]'),nl,
        write('     [ --prolog name ] [ --stdlib file.typ ] [ --backward_compatibility ]'),nl,nl,
        write('treats informations in .typ files and then type checks .pl files'),nl,nl,
        write('options:'),nl,
        write('  -I dir   add dir to the directories where tclp looks for files'),nl,
        write('  --prolog name'),nl,
        write('           tell tclp which prolog system will be used'),nl,
        write('  --stdlib file.typ'),nl,
        write('           use a custom standard library'),nl,
        write('  -v mode  enable verbose for the given mode'),nl,
        write('  -nv mode disable verbose for the given mode'),nl,
        write('      where mode can be'),nl,
        write('      actions  : prints what tclp is doing'),nl,
        write('      defaults : prints when tclp is using some defaults'),nl,
        write('      types    : prints predicates types infered or deduced'),nl,
        write('  --type_inference,'),nl,
        write('  -i       enables type inference for predicates'),nl,
        write('  --type_check'),nl,
        write('           disables type inference for predicates'),nl,
        write('  -l       enable labeling of functors type when necessary'),nl,
        write('  -nl      disable labeling of functors type'),nl,
        write('  --backward_compatibility'),nl,
        write('           enable the handling of type directives in the 0.2.x tclp format'),nl,
        write('  --help   prints this message and exits'),nl,
        nl.
        

:- include('version.pl').        

%% Run the type checker according to the command line arguments
run :-
        (
          tclp__version(Version),
          write('%% tclp '),write(Version),nl
        ;
          true
        ),
        prolog_flag(argv, Args),
        catch(chrono('all',run(Args)),
              Exception,
              (
                (Exception \= tclp__error ->
                    print_exception(Exception);
                    write('! Failed'),nl),
                halt(1)
              )
             ),
        nl,
        halt(0)
        ;
        halt(2).

user:runtime_entry(start) :- run.
